/*******************************************************************************

  Copyright (c) 2001-2003 Intel Corporation All rights reserved. 
  
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions 
  are met: 
  
  - Redistributions of source code must retain the above copyright 
  notice, this list of conditions and the following disclaimer. 
  - Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution. 
  - Neither the name of the Intel Corporation nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.
   
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INTEL 
  OR ITS  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _IXGB_H_
#define _IXGB_H_
#ifndef __IXGB_MAIN__
#define __NO_VERSION__
#endif

#include <linux/stddef.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <asm/byteorder.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/string.h>
#include <linux/pagemap.h>
#include <linux/bitops.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <net/pkt_sched.h>
#include <linux/list.h>
#include <linux/tqueue.h>
#include <linux/reboot.h>
#ifdef NETIF_F_TSO
#include <net/checksum.h>
#endif

/* ethtool support */
#ifdef SIOCETHTOOL
#include <linux/ethtool.h>
#endif
#ifdef NETIF_F_HW_VLAN_TX
#include <linux/if_vlan.h>
#endif

struct ixgb_adapter;

#define BAR_0           0
#define BAR_1           1
#define BAR_5           5 
#define PCI_DMA_64BIT   0xffffffffffffffffULL
#define PCI_DMA_32BIT   0x00000000ffffffffULL

#include "kcompat.h"

#include "ixgb_hw.h"
#include "ixgb_ee.h"
#include "ixgb_ids.h"


#if _DEBUG_DRIVER_
#define IXGB_DBG(args...) printk(KERN_DEBUG "ixgb: " args)
#else
#define IXGB_DBG(args...)
#endif

#define IXGB_ERR(args...) printk(KERN_ERR "ixgb: " args)

/* Supported Rx Buffer Sizes */
#define IXGB_RXBUFFER_2048  2048
#define IXGB_RXBUFFER_4096  4096
#define IXGB_RXBUFFER_8192  8192
#define IXGB_RXBUFFER_16384 16384

/* How many Tx Descriptors do we need to call netif_wake_queue? */
#define IXGB_TX_QUEUE_WAKE 16

/* How many Rx Buffers do we bundle into one write to the hardware ? */
#define IXGB_RX_BUFFER_WRITE    16

/* only works for sizes that are powers of 2 */
#define IXGB_ROUNDUP(i, size) ((i) = (((i) + (size) - 1) & ~((size) - 1)))

/* wrapper around a pointer to a socket buffer,
 * so a DMA handle can be stored along with the buffer */
struct ixgb_buffer {
        struct sk_buff *skb;
        uint64_t dma;
        unsigned long length;
        unsigned long time_stamp;
};

struct ixgb_desc_ring {
        /* pointer to the descriptor ring memory  */
        void *desc;
        /* physical address of the descriptor ring  */
        dma_addr_t dma;
        /* length of descriptor ring in bytes  */
        unsigned int size;
        /* number of descriptors in the ring  */
        unsigned int count;
        /* next descriptor to associate a buffer with  */
        unsigned int next_to_use;
        /* next descriptor to check for DD status bit  */
        unsigned int next_to_clean;
        /* array of buffer information structs  */
        struct ixgb_buffer *buffer_info;
};

#define IXGB_DESC_UNUSED(R) \
((((R)->next_to_clean + (R)->count) - ((R)->next_to_use + 1)) % ((R)->count))

#define IXGB_GET_DESC(R, i, type)       (&(((struct type *)((R).desc))[i]))
#define IXGB_RX_DESC(R, i)              IXGB_GET_DESC(R, i, ixgb_rx_desc)
#define IXGB_TX_DESC(R, i)              IXGB_GET_DESC(R, i, ixgb_tx_desc)
#define IXGB_CONTEXT_DESC(R, i)         IXGB_GET_DESC(R, i, ixgb_context_desc)

/* board specific private data structure */

struct ixgb_adapter {
        struct timer_list watchdog_timer;
        struct list_head proc_list_head;
#ifdef NETIF_F_HW_VLAN_TX
	struct vlan_group *vlgrp;
#endif
        char *id_string;
        uint32_t bd_number;
        uint32_t rx_buffer_len;
        uint32_t part_num;
        uint16_t link_speed;
        uint16_t link_duplex;
        atomic_t irq_sem;
        struct tq_struct tx_timeout_task;


        /* Our I/O mapped location */
        uint32_t io_base;
        /* uint32_t generate_int; */

#ifdef ETHTOOL_PHYS_ID
        struct timer_list blink_timer;
        unsigned long led_status;
#endif
#ifdef _INTERNAL_LOOPBACK_DRIVER_
        struct ixgb_desc_ring diag_tx_ring;
        struct ixgb_desc_ring diag_rx_ring;
#endif
        /* TX */
        struct ixgb_desc_ring tx_ring;
        unsigned long timeo_start;
        uint32_t tx_cmd_type;
        int max_data_per_txd;
        uint64_t hw_csum_tx_good;
        uint64_t hw_csum_tx_error;
        boolean_t tx_csum;
        uint32_t tx_int_delay;
        boolean_t tx_int_delay_enable;

        /* RX */
        struct ixgb_desc_ring rx_ring;
        uint64_t hw_csum_rx_error;
        uint64_t hw_csum_rx_good;
        uint32_t rx_int_delay;
        boolean_t raidc;
        boolean_t rx_csum;

        /* OS defined structs */
        struct net_device *netdev;
        struct pci_dev *pdev;
        struct net_device_stats net_stats;

        /* structs defined in ixgb_hw.h */
        struct ixgb_hw hw;
        struct ixgb_hw_stats stats;
        uint32_t pci_state[16];
        char ifname[IFNAMSIZ];
};

#endif /* _IXGB_H_ */
